@@ -21,12 +21,9 @@
 #include "gui/kmixerwidget.h"
 
 // Qt
-#include <QIcon>
-#include <QLabel>
 #include <qpixmap.h>
-#include <QString>
 #include <qtoolbutton.h>
-#include <QVBoxLayout>
+#include <qboxlayout.h>
 #include <qtabwidget.h>
 
 // KDE
@@ -34,14 +31,13 @@
 #include <kconfiggroup.h>
 
 // KMix
-#include "apps/kmix.h"
+#include "core/mixer.h"
+#include "core/mixertoolbox.h"
 #include "gui/guiprofile.h"
 #include "gui/kmixtoolbox.h"
 #include "gui/mixdevicewidget.h"
 #include "gui/viewsliders.h"
-#include "core/GlobalConfig.h"
-#include "core/mixer.h"
-#include "core/mixertoolbox.h"
+#include "settings.h"
 
 
 /**
@@ -61,11 +57,8 @@ KMixerWidget::KMixerWidget(Mixer *mixer,
 
 KMixerWidget::~KMixerWidget()
 {
-  foreach (ViewBase *view, _views)
-  {
-    delete view;
-  }
-  _views.clear();
+    qDeleteAll(_views);
+    _views.clear();
 }
 
 /**
@@ -91,7 +84,7 @@ void KMixerWidget::createLayout(ViewBase::ViewFlags vflags)
    GUIProfile* guiprof = getGuiprof();
    if ( guiprof != 0 )
    {
-	if (GlobalConfig::instance().data.debugGUI)
+       if (Settings::debugGui())
 		qCDebug(KMIX_LOG) << "Add a view " << _guiprofId;
     ViewSliders* view = new ViewSliders( this, guiprof->getId(), _mixer, vflags, _guiprofId, _actionCollection );
     possiblyAddView(view);
@@ -116,7 +109,7 @@ bool KMixerWidget::possiblyAddView(ViewBase* vbase)
       m_topLayout->addWidget(vbase);
       _views.push_back(vbase);
       connect( vbase, SIGNAL(toggleMenuBar()), parentWidget(), SLOT(toggleMenuBar()) );
-      if (GlobalConfig::instance().data.debugGUI)
+      if (Settings::debugGui())
     	  qCDebug(KMIX_LOG) << "CONNECT ViewBase count " << vbase->getMixers().size();
       return true;
    }
@@ -149,7 +142,7 @@ void KMixerWidget::loadConfig( KConfig *config )
     const std::vector<ViewBase*>::const_iterator viewsEnd = _views.end();
     for ( std::vector<ViewBase*>::const_iterator it = _views.begin(); it != viewsEnd; ++it) {
         ViewBase* view = *it;
-    	if (GlobalConfig::instance().data.debugVolume)
+        if (Settings::debugVolume())
     		qCDebug(KMIX_LOG) << "KMixerWidget::loadConfig()" << view->id();
         view->load(config);
         view->configurationUpdate();
@@ -164,7 +157,7 @@ void KMixerWidget::saveConfig(KConfig *config)
 	for (std::vector<ViewBase*>::const_iterator it = _views.begin(); it != viewsEnd; ++it)
 	{
 		ViewBase* view = *it;
-		if (GlobalConfig::instance().data.debugVolume)
+		if (Settings::debugVolume())
 			qCDebug(KMIX_LOG)
 			<< "KMixerWidget::saveConfig()" << view->id();
 		view->save(config);
